@@ -1,16 +1,18 @@
+from __future__ import annotations
+
 import inspect
 import json
 import os
 import sys
 import threading
-from enum import IntEnum, unique
+from enum import IntEnum
 from time import strftime
 
 global_lock = threading.Lock()
 
 
 def _merge(msg, frame: bool = True) -> str:
-    if isinstance(msg, dict) or isinstance(msg, list):
+    if isinstance(msg, (list, dict)):
         msg = f'\n{json.dumps(msg, indent=2, ensure_ascii=False)}'
     elif isinstance(msg, tuple):
         msg = " ".join([str(x).strip() for x in msg])
@@ -34,33 +36,29 @@ class LogLevel(IntEnum):
     SILENT = 4
 
 
-class Logger:
-    TRACE = LogLevel.TRACE
-    DEBUG = LogLevel.DEBUG
-    INFO = LogLevel.INFO
-    SILENT = LogLevel.SILENT
+class SingleLog:
 
-    def __init__(self, logger_name, log_level: LogLevel = INFO, handler=None, skip_repeat: bool = False,
+    def __init__(self, log_name: str, log_level: LogLevel = LogLevel.INFO, handler=None, skip_repeat: bool = False,
                  timestamp: [str | None] = "%m.%d %H:%M:%S"):
         """
         Init of SingleLog.
-        :param logger_name: the display name of current logger.
+        :param log_name: the display name of current logger.
         :param log_level: (Optional) (Default: Logger.INFO)the log level of current logger.
         :param handler: (Optional) the handler of current logger. you can get the output msg from the handler.
         :param skip_repeat: (Optional) if True, the current logger will skip the repeat msg.
         :param timestamp: (Optional) the timestamp format of current logger.
         """
 
-        self.logger_name = logger_name
-        if not self.logger_name:
-            self.logger_name = ''
+        self.log_name = log_name
+        if not self.log_name:
+            self.log_name = ''
         else:
-            self.logger_name = f'[{self.logger_name}]'
+            self.log_name = f'[{self.log_name}]'
 
         if not isinstance(log_level, LogLevel):
             raise TypeError(f'Error log level type: {type(log_level)}')
 
-        self.logger_level = log_level
+        self.log_level = log_level
 
         if handler is not None:
             if not isinstance(handler, list):
@@ -68,36 +66,34 @@ class Logger:
             for h in handler:
                 if not callable(h):
                     raise TypeError('Handler must be callable!!')
+
         self.handler = handler
         self.skip_repeat = skip_repeat
-        self.last_msg = None
         self.timestamp = timestamp
 
+        self._last_msg = None
+
     def info(self, *msg):
-        self._log(Logger.INFO, *msg)
+        self._log(LogLevel.INFO, *msg)
 
     def debug(self, *msg):
-        self._log(Logger.DEBUG, *msg)
+        self._log(LogLevel.DEBUG, *msg)
 
     def trace(self, *msg):
-        self._log(Logger.TRACE, *msg)
+        self._log(LogLevel.TRACE, *msg)
 
     def _log(self, log_level: LogLevel, *msg):
-        if self.skip_repeat:
-            if self.last_msg == msg:
-                return
-            self.last_msg = msg
-
-        if (msg_size := len(msg)) == 0:
-            msg = ' '
 
         if not isinstance(log_level, LogLevel):
             raise ValueError('Log level error')
 
-        if self.logger_level > log_level:
+        if self.log_level > log_level:
             return
 
-        if self.logger_level <= self.DEBUG:
+        if 0 == (msg_size := len(msg)):
+            msg = ' '
+
+        if self.log_level <= LogLevel.DEBUG:
             cf = inspect.currentframe()
             line_no = cf.f_back.f_back.f_lineno
             file_name = cf.f_back.f_back.f_code.co_filename
@@ -110,11 +106,17 @@ class Logger:
 
         msg = [f' {_merge(x)}' for x in msg[1:]]
         msg.insert(0, des)
+        msg = ''.join(msg)
+
+        if self.skip_repeat:
+            if self._last_msg == msg:
+                return
+            self._last_msg = msg
 
         timestamp = f'[{strftime(self.timestamp)}]' if self.timestamp else ''
         location = f'[{file_name} {line_no}]' if line_no is not None else ''
 
-        total_message = f'{timestamp}{self.logger_name}{location} {"".join(msg)}'.strip()
+        total_message = f'{timestamp}{self.log_name}{location} {msg}'.strip()
 
         with global_lock:
 
@@ -137,7 +139,15 @@ class Logger:
                     try:
                         print(total_message.encode('utf-8', "replace").decode('utf-8'))
                     except:
-                        pass
+                        print('sorry, can not print the message')
+
+
+class Logger(SingleLog):
+    # the old logger
+    TRACE = LogLevel.TRACE
+    DEBUG = LogLevel.DEBUG
+    INFO = LogLevel.INFO
+    SILENT = LogLevel.SILENT
 
 #                        ____________
 #                       |            |
